/* dcast_draw.c
   dreamcast asteroids v0.0.4: 2 players

 by
 jimbo
 Dan Potter and Jordan DeLong of Cryptic Allusion
 Andrew K.
 Marcus Comstedt

*/
//#include "dcasteroids.h" 
//#define DCASTEROIDS_USE_PVR_SPHERES

/*** courtesy Paul Boese from the scissor kgl example ***/
void rect(int x, int y, int width, int height) {
        glBegin(GL_QUADS);
        glColor4f(1,1,1,0.5f);
        glTexCoord2f(0.0f, 0.0f);
        glVertex3f(x,y,0);
        glTexCoord2f(4.0f, 0.0f);
        glVertex3f(x+width,y,0);
        glTexCoord2f(4.0f, 4.0f);
        glVertex3f(x+width,y+height,0);
        glTexCoord2f(0.0f, 4.0f);
        glVertex3f(x,y+height,0);
        glEnd();
}


/*** courtesy Dan Potter from the gltest.c example. ***/
/* Draw a cube centered around 0,0,0. Note the total lack of glTexCoord2f(),
   even though the cube is in fact getting textured! This is KGL's AUTO_UV
   feature: turn it on and it assumes you're putting on the texture on
   the quad in the order 0,0; 1,0; 1,1; 0,1. This only works for quads. */
void cube() {
    glPushMatrix();
    glScalef( 0.5, 0.5, 0.5 );
    glBegin(GL_QUADS);
    /* Front face */
    glVertex3f(-1.0f, 1.0f, 1.0f);
    glVertex3f(1.0f, 1.0f, 1.0f);
    glVertex3f(1.0f, -1.0f, 1.0f);
    glVertex3f(-1.0f, -1.0f, 1.0f);
    /* Back face */
    glVertex3f(-1.0f, -1.0f, -1.0f);
    glVertex3f(1.0f, -1.0f, -1.0f);
    glVertex3f(1.0f, 1.0f, -1.0f);
    glVertex3f(-1.0f, 1.0f, -1.0f);
    /* Left face */
    glVertex3f(-1.0f, 1.0f, 1.0f);
    glVertex3f(-1.0f, -1.0f, 1.0f);
    glVertex3f(-1.0f, -1.0f, -1.0f);
    glVertex3f(-1.0f, 1.0f, -1.0f);
    /* Right face */
    glVertex3f(1.0f, 1.0f, -1.0f);
    glVertex3f(1.0f, -1.0f, -1.0f);
    glVertex3f(1.0f, -1.0f, 1.0f);
    glVertex3f(1.0f, 1.0f, 1.0f);
    /* Top face */
    glVertex3f(1.0f, 1.0f, 1.0f);
    glVertex3f(-1.0f, 1.0f, 1.0f);
    glVertex3f(-1.0f, 1.0f, -1.0f);
    glVertex3f(1.0f, 1.0f, -1.0f);
    /* Bottom face */
    glVertex3f(1.0f, -1.0f, -1.0f);
    glVertex3f(-1.0f, -1.0f, -1.0f);
    glVertex3f(-1.0f, -1.0f, 1.0f);
    glVertex3f(1.0f, -1.0f, 1.0f);
    glEnd();
    glPopMatrix();
}



#ifdef DCASTEROIDS_USE_PVR_SHPERES


/* Draws a sphere out of GL_QUADS according to the syntax of glutSolidSphere

   This now uses triangle strips and KGL/PVR Direct Rendering for the best
   possible performance.

   Based very loosely on a routine from James Susinno.
*/
static void sphere( float radius, int slices, int stacks ) {
        int     i, j;
        float   pitch, pitch2;
        float   x, y, z, g, b;
        float   yaw, yaw2;
        pvr_vertex_t    *vert;
        pvr_dr_state_t  dr_state;

        /* Setup our Direct Render state: pick a store queue and setup QACR0/1 */
        pvr_dr_init(dr_state);

        /* Initialize xmtrx with the values from KGL */
        glKosMatrixIdent();
        glKosMatrixApply(GL_KOS_SCREENVIEW);
        glKosMatrixApply(GL_PROJECTION);
        glKosMatrixApply(GL_MODELVIEW);

        /* Put our own polygon header */
        pvr_prim(&hdr, sizeof(hdr));

        /* Iterate over stacks */
        for ( i=0; i<stacks; i++ ) {
                pitch = 2*M_PI * ( (float)i/(float)stacks );
                pitch2 = 2*M_PI * ( (float)(i+1)/(float)stacks );

                /* Iterate over slices: each entire stack will be one
                   long triangle strip. */
                for ( j=0; j<=slices/2; j++ ) {
                        yaw = 2*M_PI * ( (float)j/(float)slices );
                        yaw2 = 2*M_PI * ( (float)(j+1)/(float)slices );

                        /* x, y+1 */
                        vert = pvr_dr_target(dr_state);                 /* Get a store queue address */
                        vert->flags = PVR_CMD_VERTEX;
                        x = radius * fcos( yaw ) * fcos( pitch2 );
                        y = radius * fsin( pitch2 );
                        z = radius * fsin( yaw ) * fcos( pitch2 );
                        mat_trans_single(x, y, z);                      /* Use ftrv to transform */
                        vert->x = x;
                        vert->y = y;
                        vert->z = z;
                        vert->u = 0.0f;
                        vert->v = 0.0f;
                        g = fcos( yaw*2 ) / 2.0f + 0.5f;
                        b = fsin( phase + pitch2 ) / 2.0f + 0.5f;
                        vert->argb = 0x80000000
                                | ((uint32)(g * 255) << 8)
                                | ((uint32)(b * 255) << 0);
                        vert->oargb = 0xff000000;
                        pvr_dr_commit(vert);                            /* Prefetch the SQ */
                
                        /* x, y */
                        vert = pvr_dr_target(dr_state);
                        if (j == (slices/2))
                                vert->flags = PVR_CMD_VERTEX_EOL;
                        else
                                vert->flags = PVR_CMD_VERTEX;
                        x = radius * fcos( yaw ) * fcos( pitch );
                        y = radius * fsin( pitch );
                        z = radius * fsin( yaw ) * fcos( pitch );
                        mat_trans_single(x, y, z);
                        vert->x = x;
                        vert->y = y;
                        vert->z = z;
                        vert->u = 0.0f;
                        vert->v = 0.0f;
                        g = fcos( yaw*2 ) / 2.0f + 0.5f;
                        b = fsin( phase + pitch ) / 2.0f + 0.5f;
                        vert->argb = 0x80000000
                                | ((uint32)(g * 255) << 8)
                                | ((uint32)(b * 255) << 0);
                        vert->oargb = 0xff000000;
                        pvr_dr_commit(vert);
                }
        }
}

#else



/* draws a sphere out of GL_QUADS
 *  according to the syntax of glutSolidSphere
 *
 *
 */
static void sphere( double radius, int slices, int stacks ) {
    int i,j;

    /*** iterate over stacks ***/
    for ( i=0; i<stacks; i++ ) {
        double pitch, pitch2;
        pitch = 2.0 * 3.1415927 * ( (double)i/(double)stacks );
        pitch2 = 2.0 * 3.1415927 * ( (double)(i+1)/(double)stacks );

        /*** iterate over slices ***/
        for ( j=0; j<slices; j++ ) {
            double yaw, yaw2;
            double point[4];
            yaw = 2.0 * 3.1415927 * ( (double)j/(double)slices );
            yaw2 = 2.0 * 3.1415927 * ( (double)(j+1)/(double)slices );

            /*** draw 1 4-sided face ***/
            glBegin(GL_QUADS);

                /*** x+1, y ***/
                point[0] = radius * cos( yaw2 ) * cos( pitch );
                point[1] = radius * sin( pitch );
                point[2] = radius * sin( yaw2 ) * cos( pitch );
                glVertex3f( point[0], point[1], point[2] );

                /*** x+1, y+1 ***/
                point[0] = radius * cos( yaw2 ) * cos( pitch2 );
                point[1] = radius * sin( pitch2 );
                point[2] = radius * sin( yaw2 ) * cos( pitch2 );
                glVertex3f( point[0], point[1], point[2] );
                
                /*** x, y+1 ***/
                point[0] = radius * cos( yaw ) * cos( pitch2 );
                point[1] = radius * sin( pitch2 );
                point[2] = radius * sin( yaw ) * cos( pitch2 );
                glVertex3f( point[0], point[1], point[2] );
                
                /***  x, y ***/
                point[0] = radius * cos( yaw ) * cos( pitch );
                point[1] = radius * sin( pitch );
                point[2] = radius * sin( yaw ) * cos( pitch );
                glVertex3f( point[0], point[1], point[2] );
                
            glEnd();
        }
    }
}


#endif


/* a 3d ground
 *
 *
 *
 */
void draw_ground( float out ) {
    glPushMatrix();
    glTranslatef( -out, -out, 0 );
    glScalef( 0.5, 0.5, out );
    cube();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( out, -out, 0 );
    glScalef( 0.5, 0.5, out );
    cube();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( -out, out, 0 );
    glScalef( 0.5, 0.5, out );
    cube();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( out, out, 0 );
    glScalef( 0.5, 0.5, out );
    cube();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( 0, -out, out);
    glScalef( out, 0.5, 0.5 );
    cube();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( 0, -out, -out);
    glScalef( out, 0.5, 0.5 );
    cube();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( 0, out, out);
    glScalef( out, 0.5, 0.5 );
    cube();
    glPopMatrix();

    glPushMatrix();
    glTranslatef( 0, out, -out);
    glScalef( out, 0.5, 0.5 );
    cube();
    glPopMatrix();
}



#ifdef SHIP_MODEL_CODE
#include "dcastship.c"
#else
/* v0.0.4: soon to be replaced by an actual ship model...
 *
 *
 */
void draw_ship_shape() {
    glPushMatrix();
 
    glScalef( 1.5, 1.5, 1.5 );
    glScalef( 1.0, 0.5, 1.0 );

    glBegin(GL_QUADS);
  
    
    /* right face */
    glVertex3f(-1.0f, 1.0f, 1.0f);
    glVertex3f(1.0f, 0.2f, 0.2f);
    glVertex3f(1.0f, -0.2f, 0.2f);
    glVertex3f(-1.0f, -1.0f, 1.0f);

    /* left face */
    glVertex3f(-1.0f, -1.0f, -1.0f);
    glVertex3f(1.0f, -0.2f, -0.2f);
    glVertex3f(1.0f, 0.2f, -0.2f);
    glVertex3f(-1.0f, 1.0f, -1.0f);
    
    /* Top face */
    glVertex3f(1.0f, 0.2f, 0.1f);
    glVertex3f(-1.0f, 1.0f, 1.0f);
    glVertex3f(-1.0f, 1.0f, -1.0f);
    glVertex3f(1.0f, 0.2f, -0.1f);
    
    /* Bottom face */
    glVertex3f(1.0f, -0.2f, -0.1f);
    glVertex3f(-1.0f, -1.0f, -1.0f);
    glVertex3f(-1.0f, -1.0f, 1.0f);
    glVertex3f(1.0f, -0.2f, 0.1f);
    glEnd();
    
    glPopMatrix();
}
#endif



/* v0.0.6: draw a ufo shape
 *
 *
 *
 */
void draw_ufo_shape() {
    glPushMatrix();
        glColor3f( 0.5, 1.0, 0.5 );
        cube();
        glScalef( 1.0, 0.7, 1.0 );
        glColor3f( 1.0, 0.5, 1.0 );
        sphere( 1.0, 6, 6 );
    glPopMatrix();
}

/* v0.0.6: draw a powerup shape
 *
 *
 *
 */
void draw_powerup_shape() {
    glPushMatrix();
        cube();
    glPopMatrix();
}



/* the landscape below
 *
 *
 *
 *
 */
void draw_landscape( float bound ) {
    int i,j;
    int floorboards = 5;

    glPushMatrix();
    glTranslatef( 0.0, -bound, 0.0 );


    /*** floorboard array ***/
    glPushMatrix();
        glScalef( 10.0, 10.0, 10.0 );

        glBegin(GL_QUADS);
            for ( i= -floorboards; i<floorboards; i++ ) {
                for ( j= -floorboards; j<floorboards; j++ ) {
                    glVertex3f((float)i + 1.0f, 0.0f, (float)j + 1.0f);
                    glVertex3f((float)i + 0.0f, 0.0f, (float)j + 1.0f);
                    glVertex3f((float)i + 0.0f, 0.0f, (float)j + 0.0f);
                    glVertex3f((float)i + 1.0f, 0.0f, (float)j + 0.0f);
                }
            }
        glEnd();
        glPopMatrix();
        
    glPopMatrix();


#if 0
    /*** moon ***/
    glColor3f( 1.0, 1.0, 1.0 );
    glPushMatrix();
        glTranslatef( 450.0, 350.0, 450.0 );
        sphere( 250.0, 6, 6 );
    glPopMatrix();

    /*** red moon ***/
    glColor3f( 1.0, 0.0, 0.0 );
    glPushMatrix();
        glTranslatef( 50.0, 150.0, -350.0 );
        sphere( 150.0, 6, 6 );
    glPopMatrix();

    /*** blue moon ***/
    glColor3f( 0.0, 0.0, 1.0 );
    glPushMatrix();
        glTranslatef( -500.0, -150.0, 500.0 );
        sphere( 200.0, 6, 6 );
    glPopMatrix();
#endif

}


/* the entire code to draw the scene for any player
 *
 *
 *
 *
 */
void draw_scene( shooter_t *player ) {
    int i;
    float red, green;
    float color_scale;

#ifdef TRANSP
    // transparency?
    glEnable(GL_CULL_FACE);        
#endif


    /*** Draw in space ***/
    glLoadIdentity();



    glBindTexture(GL_TEXTURE_2D, texture );

    glColor3f(1.0f, 0.0f, 0.0f);
    if ( player->laser_charge >= MAX_LASER_CHARGE ) {
        glColor3f( 0.0f, 1.0f, 0.0f );
    }


    /*** vertical line next to crosshair ***/
    glPushMatrix();
        glTranslatef( 0.1, 0.02, -5.0 );
        glBegin(GL_QUADS);
            glVertex3f( -0.01, 0.0, 0.0 );
            glVertex3f( -0.01, (float)player->laser_charge/(float)MAX_LASER_CHARGE * 0.5, 0.0 );
            glVertex3f( 0.01, (float)player->laser_charge/(float)MAX_LASER_CHARGE *0.5, 0.0 );
            glVertex3f( 0.01, 0.0, 0.0 );
        glEnd();
    glPopMatrix();

    /*** vertical healthline next to crosshair 
     *  scale to brighten up color
     *
     */
    red = 1.0 - (float)player->hull_integrity/(float)MAX_HULL_INTEGRITY;
    green = (float)player->hull_integrity/(float)MAX_HULL_INTEGRITY;
    color_scale = player->hull_integrity < 50 ? red : green;
    glColor3f( red / color_scale, green / color_scale, 0.1 );
    glPushMatrix();
        glTranslatef( -0.1, 0.02, -5.0 );
        glBegin(GL_QUADS);
            glVertex3f( -0.02, 0.0, 0.0 );
            glVertex3f( -0.02, (float)player->hull_integrity/(float)MAX_HULL_INTEGRITY * 0.5, 0.0 );
            glVertex3f( 0.02, (float)player->hull_integrity/(float)MAX_HULL_INTEGRITY *0.5, 0.0 );
            glVertex3f( 0.02, 0.0, 0.0 );
        glEnd();
    glPopMatrix();


    if ( player->living == 0 ) {
        char pmes[256];
        /*** v0.0.6: draw player score ***/
        sprintf( pmes, "youre dead" );
        plprint( player->screen->messagetext_x, player->screen->messagetext_y, 
                 1.0f, 0.0f, 0.0f,
                 pmes, 0 );
        glKosPolyHdrDirty();
    }



    /*
     *
     * first person view - draw crosshair
     *
     *
     */
    if ( player->chosen_view == 0  ) {


        if ( player->living == 1 ) {

        /*** crosshair ***/
        glColor3f(0.0f, 1.0f, 1.0f);
        glPushMatrix();

            /*** vertical top ***/
            glBegin(GL_QUADS);
                glVertex3f( -0.008, 0.05, -5.0 );
                glVertex3f( -0.008, 0.5, -5.0 );
                glVertex3f( 0.008, 0.5, -5.0 );
                glVertex3f( 0.008, 0.05, -5.0 );
            glEnd();
            
            /*** vertical bottom ***/
            glBegin(GL_QUADS);
                glVertex3f( -0.008, -0.5, -5.0 );
                glVertex3f( -0.008, -0.05, -5.0 );
                glVertex3f( 0.008, -0.05, -5.0 );
                glVertex3f( 0.008, -0.5, -5.0 );
            glEnd();
            
            /*** horizontal left ***/
            glBegin(GL_QUADS);
                glVertex3f( -0.05, -0.008, -5.0 );
                glVertex3f( -0.5, -0.008, -5.0 );
                glVertex3f( -0.5, 0.008, -5.0 );
                glVertex3f( -0.05, 0.008, -5.0 );
            glEnd();
            
            /*** horizontal right ***/
            glBegin(GL_QUADS);
                glVertex3f( 0.5, -0.008, -5.0 );
                glVertex3f( 0.05, -0.008, -5.0 );
                glVertex3f( 0.05, 0.008, -5.0 );
                glVertex3f( 0.5, 0.008, -5.0 );
            glEnd();

        glPopMatrix();


        }
        
        /*** 1st person view ***/
        gluLookAt( player->location[0],
                   player->location[1],
                   player->location[2],
                   player->location[0] + player->direction[0],
                   player->location[1] + player->direction[1],
                   player->location[2] + player->direction[2],
                   -player->topwards[0], 
                   -player->topwards[1], 
                   -player->topwards[2]  );
                   //               0.0, 1.0, 0.0 );
    }


    /*
     *
     * external view - draw ship itself
     *
     *
     */
    else if ( player->chosen_view == 1 ) {
        float coeff = (float)(256 - player->zoom)/256.0f;
        //   float coeff = (float)(300 - player->zoom)/320.0f;
        float camera_height = -10.0 * coeff ;
        float camera_back = 28.0;

        /*** scale height for zoom ***/
        gluLookAt( player->location[0] + camera_height * player->topwards[0] - camera_back * player->direction[0],
                   player->location[1] + camera_height * player->topwards[1] - camera_back * player->direction[1],
                   player->location[2] + camera_height * player->topwards[2] - camera_back * player->direction[2],
                   player->location[0] + 10.0 * player->direction[0],
                   player->location[1] + 10.0 * player->direction[1],
                   player->location[2] + 10.0 * player->direction[2],
                   0.0, 1.0, 0.0 );

#if 1
        /*** super target ***/
        glPushMatrix();
            glTranslatef( player->location[0] + 6.0 * player->direction[0],
                          player->location[1] + 6.0 * player->direction[1],
                          player->location[2] + 6.0 * player->direction[2] );
            glScalef( 0.1, 0.1, 0.1 );
            glColor3f(1.0,1.0,1.0);
            cube();
        glPopMatrix();
#endif
    }




    /*** the world around ***/
    glPushMatrix();    
        glColor3f(1.0,1.0,1.0);
        draw_ground( bound_size );
        glColor3f(1.0,1.0,1.0);
        draw_ground( 2.0 * bound_size );        
        /*** earth ***/
        glColor3f( 0.27, 0.039, 0.0 );
        glBindTexture(GL_TEXTURE_2D, texture5 );
        draw_landscape( 2.0 * bound_size );
        glScalef( 1.0, -1.0, 1.0 );
        glColor3f( 0.121, 0.760, 0.96  );
        glBindTexture(GL_TEXTURE_2D, texture6 );
        draw_landscape( 2.0 * bound_size );
    glPopMatrix();




    /* draw players
     *
     *
     */
    for ( i=0; i<player_count; i++ ) {
        if ( players[i]->living == 1 ) {
            if ( ( player->chosen_view != 0 ) || ( players[i] != player ) ) {
                //  glColor3f(0.01,0.10,1.0);
                glColor3fv( players[i]->color );

                glPushMatrix();
                glTranslatef( players[i]->location[0], 
                              players[i]->location[1], 
                              players[i]->location[2] );

                /*** line up with shooting position ***/
                glRotatef( -180.0/3.1415927 * players[i]->yaw, 0.0, 1.0, 0.0 );
                glRotatef( 180.0/3.1415927 * players[i]->pitch, 0.0, 0.0, 1.0 );

                draw_ship_shape();
                glPopMatrix();
            }   
        }    

    }



    /* draw asteroids
     *
     *
     */
    glColor3f( 0.5, 0.5, 0.5 );
    glBindTexture(GL_TEXTURE_2D, texture3 );
    for ( i=0; i<asteroid_count; i++ ) {
        int slice = asteroids[i]->size + 2;
        if ( slice == 4 ) { slice++; }

        glPushMatrix();
            glTranslatef( asteroids[i]->location[0], 
                          asteroids[i]->location[1], 
                          asteroids[i]->location[2] );
            glRotatef( asteroids[i]->rotation[0], 1.0, 0.0, 0.0 );
            glRotatef( asteroids[i]->rotation[1], 0.0, 1.0, 0.0 );
//        glRotatef( asteroids[i]->rotation[2], 0.0, 0.0, 1.0 );

            /* v0.0.5: what is going on here?
             *  one line is fine, then used a locally scoped slice and crash.
             *
             */
            // sphere( asteroids[i]->radius, asteroids[i]->size + 2, asteroids[i]->size + 2 );
               sphere( asteroids[i]->radius, slice, slice );

        glPopMatrix();
    }    
    



    /* draw moving shots
     *
     *
     */ 
    glColor3f(0.8,0.8,1.0);
    for ( i=0; i<active_moving_shot_count; i++ ) {

        glPushMatrix();
        glTranslatef( active_moving_shots[i]->location[0], 
                      active_moving_shots[i]->location[1], 
                      active_moving_shots[i]->location[2] );

        glScalef(0.2, 0.2, 0.2 );
        cube();
        glPopMatrix();

    }




    /* v0.0.6: draw enemies
     *
     *
     */
    for ( i=0; i<enemy_count; i++ ) {

        glPushMatrix();
            glTranslatef( enemies[i]->location[0], 
                          enemies[i]->location[1], 
                          enemies[i]->location[2] );
            draw_ufo_shape();
        glPopMatrix();

    }



    /* v0.0.6: draw enemies
     *  0 - health 
     *  1 - machine gun (red)
     *  2 - spread     (blue)
     * 
     *
     */
    glBindTexture(GL_TEXTURE_2D, texture );
    for ( i=0; i<powerup_count; i++ ) {
       switch( powerups[i]->type ) {
       case 0: glColor3f( 1.0, 1.0, 1.0 ); break;
       case 1: glColor3f( 1.0, 0.0, 0.0 ); break;
       case 2: glColor3f( 0.0, 0.0, 1.0 ); break;
       default: glColor3f( 0.0, 1.0, 1.0 ); break;
       }
        glPushMatrix();
            glTranslatef( powerups[i]->location[0], 
                          powerups[i]->location[1], 
                          powerups[i]->location[2] );
            glRotatef( powerups[i]->rotation[0], 1.0, 0.0, 0.0 );
            glRotatef( powerups[i]->rotation[1], 0.0, 1.0, 0.0 );
            draw_powerup_shape();
        glPopMatrix();

    }



    /* v0.0.5: draw explosions
     *Potentially do two as translucent
     *
     *
     */
#ifdef TRANSP
    if ( 1 ) {
        glKosFinishList();
        // glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
        glDisable(GL_CULL_FACE);
    }
#endif



    glBindTexture(GL_TEXTURE_2D, texture2 );
#ifdef TRANSP
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_FILTER, GL_FILTER_BILINEAR);
    glTexEnvi(GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATEALPHA);

#endif
    for ( i=0; i<explosion_count; i++ ) {
        float siz = (float)explosions[i]->lifetime / (float)MAX_EXPLOSION_LIFETIME;
        float col = 1.0 - siz;
        glPushMatrix();
            glTranslatef( explosions[i]->location[0], 
                          explosions[i]->location[1], 
                          explosions[i]->location[2] );
#ifdef TRANSP
            glColor4f( col * 0.9, col * 0.5, col * 0.0, 0.5 );
#else
            glColor3f( col * 0.9, col * 0.5, col * 0.0 );
#endif
            glScalef( 2.0, 2.0, 2.0 );
            glScalef( explosions[i]->radius, explosions[i]->radius, explosions[i]->radius );
            glScalef( siz, siz, siz );
            //  cube();
            sphere( 1.0, 6, 6 );
        glPopMatrix();
    }
#ifdef TRANSP
    if (1) {
        glEnable(GL_CULL_FACE);
    }
#endif



    /* draw laser shots (static)
     *  - should be a line drawing
     *
     */ 
    for ( i=0; i<active_shot_count; i++ ) {
        float inten = (float)( active_shots[i]->potent )/(float)MAX_SHOT_POTENCY;
        //glColor3f( 0.3 * inten, 0.0, inten );
        glColor4f( 0.3 * inten, 0.0, inten, 0.75 );
        glBegin(GL_QUADS);
            glVertex3f( active_shots[i]->origin[0], 
                        active_shots[i]->origin[1],
                        active_shots[i]->origin[2] );
            glVertex3f( active_shots[i]->origin[0] + 100.0 * active_shots[i]->direction[0], 
                        active_shots[i]->origin[1] + 100.0 * active_shots[i]->direction[1],
                        active_shots[i]->origin[2] + 100.0 * active_shots[i]->direction[2] );
            glVertex3f( active_shots[i]->origin[0] + 100.0 * active_shots[i]->direction[0], 
                        active_shots[i]->origin[1] + 100.0 * active_shots[i]->direction[1] + 0.05,
                        active_shots[i]->origin[2] + 100.0 * active_shots[i]->direction[2] );
            glVertex3f( active_shots[i]->origin[0], 
                        active_shots[i]->origin[1] + 0.05,
                        active_shots[i]->origin[2] );
        glEnd();

    }







    /* draw preference screen at the end
     *
     *
     *
     */
    if ( player->pausing == 1 ) {

        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        gluPerspective(30.0f, 640.0f / 320.0f, 0.01f, 1000.0f);

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        gluLookAt( 0, 2, 15, 0, 1, 0, 0, 1, 0 );

        glTranslatef( 0.0, 2.0, 0.0 );

        glColor3f(0.8,0.8,1.0);
        glPushMatrix();




            /*** line up with end of screen ***/
            glTranslatef( -7.0, 0.0, 0.0 );


            /*** green - sensitivity ***/
            glPushMatrix();

              glTranslatef( 0.0, 2.0, 0.0 );
              glColor3f( 0.0, 1.0, 0.0 );
              /*** face ***/
              glScalef( 5.0 * 0.05 * (float)player->sensitivity_wide, 0.6, 1.0 );
              glBegin(GL_QUADS);
                glVertex3f(0.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 0.0f, 1.0f);
                glVertex3f(0.0f, 0.0f, 1.0f);
              glEnd();
            
            glPopMatrix();




            /*** yellow - zoom sensitivity ***/
            glPushMatrix();

              glTranslatef( 0.0, 1.0, 0.0 );
              glColor3f( 1.0, 1.0, 0.0 );
              /*** face ***/
              glScalef( 5.0 * 0.05 * (float)player->sensitivity_tight, 0.6, 1.0 );
              glBegin(GL_QUADS);
                glVertex3f(0.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 0.0f, 1.0f);
                glVertex3f(0.0f, 0.0f, 1.0f);
              glEnd();

            glPopMatrix();





            /*** red - regular fov ***/
            glPushMatrix();

            // glTranslatef( 0.0, 0.0, 0.0 );
              glColor3f( 0.0, 0.0, 1.0 );
              /*** face ***/
              glScalef( 5.0 * 0.05 * (float)player->fov_wide, 0.6, 1.0 );
              glBegin(GL_QUADS);
                glVertex3f(0.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 0.0f, 1.0f);
                glVertex3f(0.0f, 0.0f, 1.0f);
              glEnd();

            glPopMatrix();



            /*** blue - zoom fov ***/
            glPushMatrix();

             glTranslatef( 0.0, -1.0, 0.0 );
              glColor3f( 1.0, 0.0, 0.0 );
              /*** face ***/
              glScalef( 5.0 * 0.05 * (float)player->fov_tight, 0.6, 1.0 );
              glBegin(GL_QUADS);
                glVertex3f(0.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 1.0f, 1.0f);
                glVertex3f(1.0f, 0.0f, 1.0f);
                glVertex3f(0.0f, 0.0f, 1.0f);
              glEnd();

            glPopMatrix();



        glPopMatrix();
    }

}




