/* dcast_main.c
   dreamcast asteroids v0.0.6: more stuff

 by
 jimbo
 Dan Potter and Jordan DeLong of Cryptic Allusion
 Andrew K.
 Marcus Comstedt


   something to remember: these libs use negative radians in glRotatef()
    relative to my linux/NVidia GL libs.


   using
   KallistiOS 1.1.6

   adapted from
   gltest.c
   (c)2001 Dan Potter
*/


#include "dcasteroids.h"



/*** for text display ***/
#include "dcf_draw.c"
#include "plprint.h"




/*** global registers ***/
shot_t *active_shots[ MAX_SHOTS ];
int active_shot_count = 0;

moving_shot_t *active_moving_shots[ MAX_SHOTS ];
int active_moving_shot_count = 0;

asteroid_t *asteroids[ MAX_ASTEROIDS ];
int asteroid_count = 0;

explosion_t *explosions[ MAX_SHOTS ];
int explosion_count = 0;

shooter_t *players[4];
int player_count = 0;

enemy_t *enemies[ MAX_ENEMIES ];
int enemy_count = 0;

powerup_t *powerups[ MAX_ENEMIES ];
int powerup_count = 0;


int total_shot_count = 0;

double bound_size = 20.0;



GLuint texture, texture2, texture3;
GLuint texture4, texture5 ,texture6;


/*** v0.0.5: levels ***/
int current_level = 1;
/*** v0.0.6: frequency of random items ***/
int enemy_frequency = 10;
int enemy_giveup = 100;
int powerup_frequency = 10;


#ifdef DCASTEROIDS_USE_PVR_SPHERES
static float phase = 0.0f;
static pvr_poly_cxt_t cxt;
static pvr_poly_hdr_t hdr;
#endif

/*** dcast_control.c refers to some(all?) of the above globals ***/
#include "dcast_control.c"
#include "dcast_draw.c"
#include "dcast_consequence.c"



/* start location, etc.
 *  for player 1
 *
 *
 */
void init_shooter( shooter_t *s ) {

    s->location[0] = 0.0;
    s->location[1] = 0.0; 
    s->location[2] = 0.0;

    s->velocity[0] = 0.0;
    s->velocity[1] = 0.0;
    s->velocity[2] = 0.0;

    s->rotation[0] = 0.0;
    s->rotation[1] = 0.0;
    s->rotation[2] = 0.0;

    /*** this is calculated from yaw and pitch and roll ***/
    s->direction[0] = 0.0;
    s->direction[1] = 0.0;
    s->direction[2] = 1.0;

    s->topwards[0] = 0.0;
    s->topwards[1] = 0.0;
    s->topwards[2] = 0.0;

    s->pitch = 0.0;
    s->yaw = 0.0;
    s->roll = 0.0;

    s->zoom = 0;
    s->chosen_view = 0;
    s->laser_charge = 0;

    s->damage_hits = 0;
    s->hull_integrity = 100;
    s->score = 0;
    s->shots_fired = 0;
    s->shots_hit = 0;


    

    s->braking = 0;
    s->pausing = 0;
    s->living = 1;
    s->machine_gun = 0;
    s->spread_gun = 0;

    s->chosen_option = 0;

    s->sensitivity_tight = 2;
    s->sensitivity_wide = 10;
    s->fov_tight = 10;
    s->fov_wide = 50;



    s->last_ltrig = 0;


    s->screen = NULL;
}

/* initialize an enemy
 *
 *
 *
 */
void init_enemy( enemy_t *e ) {
    double sum = 0.0;

    e->location[0] = bound_size * (float)(rand()%100)/50.0f;
    e->location[1] = bound_size * (float)(rand()%100)/50.0f;
    e->location[2] = bound_size * (float)(rand()%100)/50.0f;

    // point to player 1 first
    e->direction[0] = players[0]->location[0] - e->location[0];
    e->direction[1] = players[0]->location[1] - e->location[1];
    e->direction[2] = players[0]->location[2] - e->location[2];

    e->velocity[0] = players[0]->location[0] - e->location[0];
    e->velocity[1] = players[0]->location[1] - e->location[1];
    e->velocity[2] = players[0]->location[2] - e->location[2];

    // normalize
    sum += e->velocity[0] * e->velocity[0];
    sum += e->velocity[1] * e->velocity[1];
    sum += e->velocity[2] * e->velocity[2];
    sum = sqrt( sum );
    
    sum *= 100.0;

    e->velocity[0] /= sum;
    e->velocity[1] /= sum;
    e->velocity[2] /= sum;
    e->direction[0] /= sum;
    e->direction[1] /= sum;
    e->direction[2] /= sum;

    e->hull_integrity = 100;
    e->shot_cooldown = 0;
    e->lifetime = 0;

    //   e->opponent = NULL;
    e->opponent = players[0];
}

/* initialize a full, complete asteroid randomly.
 *
 *
 *
 */
void init_asteroid( asteroid_t *ast ) {

    ast->location[0] = bound_size * (float)(rand()%100 - 50)/50.0f;
    ast->location[1] = bound_size * (float)(rand()%100 - 50)/50.0f;
    ast->location[2] = bound_size * (float)(rand()%100 - 50)/50.0f;

    ast->rotation[0] = (float)(rand()%100 - 50)/500.0f;
    ast->rotation[1] = (float)(rand()%100 - 50)/500.0f;
    ast->rotation[2] = (float)(rand()%100 - 50)/500.0f;

    ast->rotation_speed[0] = (float)(rand()%100 - 50)/5.0f;
    ast->rotation_speed[1] = (float)(rand()%100 - 50)/5.0f;
    ast->rotation_speed[2] = (float)(rand()%100 - 50)/5.0f;

    ast->velocity[0] = (float)(rand()%100 - 50)/1000.0f;
    ast->velocity[1] = (float)(rand()%100 - 50)/1000.0f;
    ast->velocity[2] = (float)(rand()%100 - 50)/1000.0f;

    ast->size = 4;
    ast->radius = 4.0;
}


/* initialize a full, complete asteroid randomly.
 *  0 - health
 *  1 - machine gun
 *  2 - spread
 *
 *
 */
void init_powerup( powerup_t *pow ) {

    pow->location[0] = bound_size * (float)(rand()%100 - 50)/50.0f;
    pow->location[1] = bound_size * (float)(rand()%100 - 50)/50.0f;
    pow->location[2] = bound_size * (float)(rand()%100 - 50)/50.0f;

    pow->rotation[0] = (float)(rand()%100 - 50)/500.0f;
    pow->rotation[1] = (float)(rand()%100 - 50)/500.0f;
    pow->rotation[2] = (float)(rand()%100 - 50)/500.0f;

    pow->rotation_speed[0] = (float)(rand()%100 - 50)/5.0f;
    pow->rotation_speed[1] = (float)(rand()%100 - 50)/5.0f;
    pow->rotation_speed[2] = (float)(rand()%100 - 50)/5.0f;

    pow->velocity[0] = (float)(rand()%100 - 50)/1000.0f;
    pow->velocity[1] = (float)(rand()%100 - 50)/1000.0f;
    pow->velocity[2] = (float)(rand()%100 - 50)/1000.0f;

    /*** number of types of powerups ***/
    pow->type = rand()%3;
    pow->radius = 4.0;
}




/* v0.0.5: create an explosion from the shot
 *
 *
 */
void create_explosion( moving_shot_t *sho ) {
    explosion_t *exp;
    if ( explosion_count >= MAX_SHOTS ) { return; }

    exp = (explosion_t*)malloc( sizeof( explosion_t ) );
    exp->location[0] = sho->location[0];
    exp->location[1] = sho->location[1];
    exp->location[2] = sho->location[2];
    exp->lifetime = MIN_EXPLOSION_LIFETIME;

    exp->radius = 1.0;

    explosions[ explosion_count++ ] = exp;
}

void create_enemy_explosion( enemy_t *en ) {
    explosion_t *exp;
    if ( explosion_count >= MAX_SHOTS ) { return; }

    exp = (explosion_t*)malloc( sizeof( explosion_t ) );
    exp->location[0] = en->location[0];
    exp->location[1] = en->location[1];
    exp->location[2] = en->location[2];
    exp->lifetime = MIN_EXPLOSION_LIFETIME;

    exp->radius = 2.0;

    explosions[ explosion_count++ ] = exp;
}

void create_player_explosion( shooter_t *pl ) {
    explosion_t *exp;
    if ( explosion_count >= MAX_SHOTS ) { return; }

    exp = (explosion_t*)malloc( sizeof( explosion_t ) );
    exp->location[0] = pl->location[0];
    exp->location[1] = pl->location[1];
    exp->location[2] = pl->location[2];
    exp->lifetime = MIN_EXPLOSION_LIFETIME;

    exp->radius = 3.0;

    explosions[ explosion_count++ ] = exp;
}


/* v0.0.5: reset between levels
 *
 *  clear all arrays
 *
 */
void clear_game() {
    int i,j,k;
 

    for ( i=0; i<asteroid_count; i++ ) {
        free( asteroids[i] );
        for ( j=i; j<asteroid_count; j++ ) {
            asteroids[j] = asteroids[j+1];
        }
        asteroid_count--;
    }

    for ( i=0; i<active_shot_count; i++ ) {
        free( active_shots[i] );
        for ( j=i; j<active_shot_count; j++ ) {
            active_shots[j] = active_shots[j+1];
        }
        active_shot_count--;
    }



    for ( i=0; i<active_moving_shot_count; i++ ) {
        free( active_moving_shots[i] );
        for ( j=i; j<active_moving_shot_count; j++ ) {
            active_moving_shots[j] = active_moving_shots[j+1];
        }
        active_moving_shot_count--;
    }


    for ( i=0; i<explosion_count; i++ ) {
        free( explosions[i] );
        for ( j=i; j<explosion_count; j++ ) {
            explosions[j] = explosions[j+1];
        }
        explosion_count--;
    }

#if 0
    for ( i=0; i<enemy_count; i++ ) {
        free( enemies[i] );
        for ( j=i; j<enemy_count; j++ ) {
            enemies[j] = enemies[j+1];
        }
        enemy_count--;
    }
#endif

}


/* v0.0.5: the level decider
 *
 *  put the first asteroids in
 *
 */
void init_game_level( int level ) {
    int i;

    switch( level ) {

        /*** level 1 ***/
    case 1:
        for ( i=0; i<1; i++ ) {
            asteroids[i] = malloc( sizeof( asteroid_t ) );
            init_asteroid( asteroids[i] );
            asteroid_count++;
            powerup_frequency = 100;
            enemy_frequency = 1;
            enemy_giveup = 100;
        }


        break;


    case 2:
        for ( i=0; i<2; i++ ) {
            asteroids[i] = malloc( sizeof( asteroid_t ) );
            init_asteroid( asteroids[i] );
            asteroid_count++;
            powerup_frequency = 50;
            enemy_frequency = 5;
            enemy_giveup = 200;
        }
        break;



        /*** all others ***/
    default:
        for ( i=0; i<3; i++ ) {
            asteroids[i] = malloc( sizeof( asteroid_t ) );
            init_asteroid( asteroids[i] );
            asteroid_count++;
            enemy_frequency = 10 + level;
            powerup_frequency = (int)( 35.0/(float)level );
            enemy_giveup = 300 + 10 * level;
        }
        break;
    }


    




}



/* the scores for each asteroid type
 *
 *
 */
int size_to_score( int size ) {
    switch( size ) {
    case 1: return 20; break;
    case 2: return 10; break;
    case 3: return 5; break;
    case 4: return 2; break;
    default: return 0; break;
    }
    return 0;
}


/*** Dan Potter's code ***/
/* Load a texture using pcx_load_texture and glKosTex2D */
void loadtxr(const char *fn, GLuint *txr) {
    uint32 txaddr, w, h;
    if (pcx_load_texture(fn, 1, 0, &txaddr, &w, &h) < 0) {
        printf("can't load %s\n", fn);
        return;
    }

    glGenTextures(1, txr);
    glBindTexture(GL_TEXTURE_2D, *txr);
    glKosTex2D(GL_RGB565_TWID, w, h, txaddr);
}
/*** from the kgl/tunnel demo ***/
pvr_init_params_t params = {
       /* Enable opaque and punchthru polygons with size 16 */
       { PVR_BINSIZE_32, PVR_BINSIZE_0, PVR_BINSIZE_32, PVR_BINSIZE_0, PVR_BINSIZE_0 },

        /* Vertex buffer size 512K */
        512*1024
};

extern uint8 romdisk[];



/*** v0.0.6: radar screen ***/
void draw_radar_screen() {
    int i;

    glDisable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);
    glDepthFunc(GL_LESS);

    /*** draw asteroids from above ***/
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    //glOrtho(0,640,0,480,-1,1);
    gluLookAt( 0, 200, 0,   0, 0, 0,   0, 0, 1 );
            
    glColor3f (1.0, 0.0, 0.0 );

    //   glClearColor( 0.2, 0.3, 0.5, 0.5 );
    glPushMatrix();
    glTranslatef( 0.0, -2.0, 0.0 );
    glScalef( 10, 10, 10 );
    glColor3f( 0.1, 0.1, 0.1 );
                
    glBegin(GL_QUADS);
    glVertex3f(1.0f, 1.0f, 1.0f);
    glVertex3f(-1.0f, 1.0f, 1.0f);
    glVertex3f(-1.0f, 1.0f, -1.0f);
    glVertex3f(1.0f, 1.0f, -1.0f);
    glEnd();
    glPopMatrix();

    glColor3f( 1.0, 0.5, 0.1 );

    for ( i=0; i<asteroid_count; i++ ) {
        glPushMatrix();
        glTranslatef( asteroids[i]->location[0], 
                      0.0,  
                      asteroids[i]->location[2] );
        // sphere( 1.0, 6, 6 );
                    
        glBegin(GL_QUADS);
        glVertex3f(1.0f, 1.0f, 1.0f);
        glVertex3f(-1.0f, 1.0f, 1.0f);
        glVertex3f(-1.0f, 1.0f, -1.0f);
        glVertex3f(1.0f, 1.0f, -1.0f);
        glEnd();

        glPopMatrix();

    }

}




/*
 *
 *
 *
 */
int main(int argc, char **argv) {
    int i;
    int player_devices[4][2];
    /*** Dan Potter's variables ***/
    cont_cond_t conds[4];
    uint8	cs[4];

    char pmes[256];


    int running_state = RUNSTATE_PLAYING;
    int transpause = 0;
    int transview = 0;

    int textbar_height = 0;

    /*** v0.2.2: frame counter ***/
#ifdef FRAME_RATE_COUNTER
    int frame_count = 0;
    time_t begin_time;
    time_t last_time;
    time_t time_value;
    double frames_per_second;
#endif

    subscreen_t *info_band;


    /*** Dan Potter's code ***/
    /* Initialize KOS */
    kos_init_all(THD_ENABLE | IRQ_ENABLE | TA_ENABLE, romdisk); 
    pvr_init(&params);

    /* Get basic stuff initialized */
    glKosInit();
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // gluPerspective(30.0f, 640.0f / 480.0f, 0.01f, 1000.0f);
    //gluPerspective(30.0f, 640.0f / 240.0f, 0.01f, 1000.0f);
    glMatrixMode(GL_MODELVIEW);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_KOS_AUTO_UV);

    glEnable(GL_SCISSOR_TEST);
  
#ifdef TRANSP
    /* Enable Transparancy */
    //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    /* Enable Transparancy */
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glEnable(GL_DEPTH_TEST);


#endif

#ifdef DCASTEROIDS_USE_PVR_SPHERES
    /*** v0.0.6: from the bubbles demo ***/
    pvr_poly_cxt_col(&cxt, PVR_LIST_OP_POLY);
    cxt.gen.culling = PVR_CULLING_NONE;
    pvr_poly_compile(&hdr, &cxt);
#endif

    /* v0.0.2: init text services
     * v0.0.3:  and sound
     *PLIB for fonts
     */
    draw_init();
    plinit();



    /* Load a texture */
    loadtxr("/rd/crate.pcx", &texture);
    loadtxr("/rd/flames.pcx", &texture2);
    loadtxr("/rd/greystone.pcx", &texture3);
    loadtxr("/rd/leaves.pcx", &texture5);
    loadtxr("/rd/cloud.pcx", &texture6);



#ifdef SOUND_EFFECTS
    /*** sound init ***/
    snd_stream_init( NULL );
    snd_sfx_load( "/rd/Hit_m.wav" );
    snd_sfx_load( "/rd/Explode.wav" );
    snd_sfx_load( "/rd/Laser4.wav" );
    snd_sfx_load( "/rd/Laser5.wav" );
    snd_sfx_load( "/rd/Phaser03.wav" );
    snd_sfx_load( "/rd/Ricchet2.wav" );
    snd_sfx_load( "/rd/Hit_l.wav" );
    snd_sfx_load( "/rd/blip.wav" );
    snd_sfx_load( "/rd/boink.wav" );
    snd_sfx_load( "/rd/dink.wav" );
    snd_sfx_load( "/rd/drip.wav" );
    snd_sfx_load( "/rd/rise.wav" );
    snd_sfx_load( "/rd/whistle.wav" );



#endif

    // mp3_init();
#ifdef OGGVORBIS_BGM
    sndoggvorbis_init();
    sndoggvorbis_start( "/rd/vectorman.ogg", 1 );
#endif






    /* where's the second controller?
     *  Input device check - scan bus
     *
     *
     */
    for ( i=0; i<4; i++ ) {
        int j;
        for ( j=0; j<4; j++ ) {
            printf("Device %d %d) %d\n", i, j, maple_device_func(i,j) );

            /*** found controller (thanks Dan) ***/
            if ( maple_device_func(i,j) == MAPLE_FUNC_CONTROLLER ) {
                /*** player ***/
                player_devices[ player_count ][0] = i;
                player_devices[ player_count ][1] = j;


                cs[player_count] = maple_create_addr( player_devices[player_count][0] , player_devices[player_count][1] );
                players[player_count] = malloc( sizeof( shooter_t ) );
                init_shooter( players[player_count] );

                player_count++;
            }

        }
    }


    init_game_level( current_level );


   

    /*** player 1 - red ***/
    if ( player_count >= 1 ) {
        players[0]->location[0] = -20.0;
        players[0]->location[1] = 5.0; 
        players[0]->location[2] = 20.0;
        players[0]->yaw = -3.1415927/4.0;
        players[0]->color[0] = 1.0;
        players[0]->color[1] = 0.0;
        players[0]->color[2] = 0.0;
    }
    /*** p2 blue ***/
    if ( player_count >= 2 ) {
        players[1]->location[0] = 20.0;
        players[1]->location[1] = 5.0; 
        players[1]->location[2] = 20.0;
        players[1]->yaw = 3.1415927 + 3.1415927/4.0;
        players[1]->color[0] = 0.0;
        players[1]->color[1] = 0.0;
        players[1]->color[2] = 1.0;
    }
    /*** p3 yellow ***/
    if ( player_count >= 3 ) {
        players[2]->location[0] = 20.0;
        players[2]->location[1] = 5.0; 
        players[2]->location[2] = -20.0;
        players[2]->yaw = 3.1415927/2.0 + 3.1415927/4.0;
        players[2]->color[0] = 1.0;
        players[2]->color[1] = 1.0;
        players[2]->color[2] = 0.0;
    }
    /*** p4 green ***/
    if ( player_count >= 4 ) {
        players[3]->location[0] = -20.0;
        players[3]->location[1] = 5.0; 
        players[3]->location[2] = -20.0;
        players[3]->yaw = 3.1415927/4.0;
        players[3]->color[0] = 0.0;
        players[3]->color[1] = 1.0;
        players[3]->color[2] = 0.0;
    }




    /* v0.0.6: set screen sizes at init time 
     *
     *
     *
     */
    info_band = malloc( sizeof( subscreen_t ) );
    if ( player_count == 1 ) {
        players[0]->screen = malloc( sizeof( subscreen_t ) );
        players[0]->screen->x = 0;
        players[0]->screen->y = 0;
        players[0]->screen->width = 640;
        players[0]->screen->height = 448;
        players[0]->screen->scoretext_x = 32;
        players[0]->screen->scoretext_y = 2;
        players[0]->screen->messagetext_x = 5;
        players[0]->screen->messagetext_y = 5;

        info_band->x = 0;
        info_band->y = 448;
        info_band->width = 640;
        info_band->height = 32;
        info_band->messagetext_x = 2;
        info_band->messagetext_y = 30;
    }

    else if ( player_count == 2 ) {
        players[0]->screen = malloc( sizeof( subscreen_t ) );
        players[0]->screen->x = 0;
        players[0]->screen->y = 252;
        players[0]->screen->width = 640;
        players[0]->screen->height = 224;
        players[0]->screen->scoretext_x = 32;
        players[0]->screen->scoretext_y = 2;
        players[0]->screen->messagetext_x = 5;
        players[0]->screen->messagetext_y = 5;

        players[1]->screen = malloc( sizeof( subscreen_t ) );
        players[1]->screen->x = 0;
        players[1]->screen->y = 0;
        players[1]->screen->width = 640;
        players[1]->screen->height = 224;
        players[1]->screen->scoretext_x = 32;
        players[1]->screen->scoretext_y = 28;
        players[1]->screen->messagetext_x = 5;
        players[1]->screen->messagetext_y = 20;

        info_band->x = 0;
        info_band->y = 224;
        info_band->width = 640;
        info_band->height = 32;
        info_band->messagetext_x = 2;
        info_band->messagetext_y = 15;
    }

    else if ( player_count == 3 ) {
        players[0]->screen = malloc( sizeof( subscreen_t ) );
        players[0]->screen->x = 0;
        players[0]->screen->y = 252;
        players[0]->screen->width = 640;
        players[0]->screen->height = 224;
        players[0]->screen->scoretext_x = 32;
        players[0]->screen->scoretext_y = 2;
        players[0]->screen->messagetext_x = 5;
        players[0]->screen->messagetext_y = 5;

        players[1]->screen = malloc( sizeof( subscreen_t ) );
        players[1]->screen->x = 0;
        players[1]->screen->y = 0;
        players[1]->screen->width = 320;
        players[1]->screen->height = 224;
        players[1]->screen->scoretext_x = 2;
        players[1]->screen->scoretext_y = 28;
        players[1]->screen->messagetext_x = 5;
        players[1]->screen->messagetext_y = 20;

        players[2]->screen = malloc( sizeof( subscreen_t ) );
        players[2]->screen->x = 320;
        players[2]->screen->y = 0;
        players[2]->screen->width = 320;
        players[2]->screen->height = 224;
        players[2]->screen->scoretext_x = 32;
        players[2]->screen->scoretext_y = 28;
        players[2]->screen->messagetext_x = 25;
        players[2]->screen->messagetext_y = 20;

        info_band->x = 0;
        info_band->y = 224;
        info_band->width = 640;
        info_band->height = 32;
        info_band->messagetext_x = 2;
        info_band->messagetext_y = 15;
    }


    else if ( player_count == 4 ) {
        players[0]->screen = malloc( sizeof( subscreen_t ) );
        players[0]->screen->x = 0;
        players[0]->screen->y = 252;
        players[0]->screen->width = 320;
        players[0]->screen->height = 224;
        players[0]->screen->scoretext_x = 2;
        players[0]->screen->scoretext_y = 2;
        players[0]->screen->messagetext_x = 5;
        players[0]->screen->messagetext_y = 5;

        players[1]->screen = malloc( sizeof( subscreen_t ) );
        players[1]->screen->x = 320;
        players[1]->screen->y = 224;
        players[1]->screen->width = 320;
        players[1]->screen->height = 224;
        players[1]->screen->scoretext_x = 32;
        players[1]->screen->scoretext_y = 2;
        players[1]->screen->messagetext_x = 25;
        players[1]->screen->messagetext_y = 5;

        players[2]->screen = malloc( sizeof( subscreen_t ) );
        players[2]->screen->x = 0;
        players[2]->screen->y = 0;
        players[2]->screen->width = 320;
        players[2]->screen->height = 224;
        players[2]->screen->scoretext_x = 2;
        players[2]->screen->scoretext_y = 28;
        players[2]->screen->messagetext_x = 5;
        players[2]->screen->messagetext_y = 20;


        players[3]->screen = malloc( sizeof( subscreen_t ) );
        players[3]->screen->x = 320;
        players[3]->screen->y = 0;
        players[3]->screen->width = 320;
        players[3]->screen->height = 224;
        players[3]->screen->scoretext_x = 32;
        players[3]->screen->scoretext_y = 28;
        players[3]->screen->messagetext_x = 25;
        players[3]->screen->messagetext_y = 20;

        info_band->x = 0;
        info_band->y = 224;
        info_band->width = 640;
        info_band->height = 32;
        info_band->messagetext_x = 2;
        info_band->messagetext_y = 15;
    }








#ifdef FRAME_RATE_COUNTER
    timer_init();
    begin_time = rtc_unix_secs();
#endif 
  
    /* MAIN LOOP:
     *  - get controller input (possibly button combos)
     *  
     *  
     *
     *  - call consequence functions
     *
     *  - draw it from player/radar perspectives
     *
     */
    while(1) {

#ifdef FRAME_RATE_COUNTER
        int tim = (time_value & 0xFFFF) - (begin_time & 0xFFFF);
        /* v0.2.2: update frame rate counter
         *    - courtesy of Trilinear
         *
         */
        time_value = rtc_unix_secs();
        if ( time_value != last_time ) {
            // int tim = (time_value & 0xFFFF) - (begin_time & 0xFFFF);
            last_time = time_value;
            if ( tim > 0 ) {
                frames_per_second = (double)frame_count/(double)tim;
            }
            else frames_per_second = 0.0;
        }
#endif

        /*** read controllers ***/
        for ( i=0; i<player_count; i++ ) {
            if (cont_get_cond(cs[i], &conds[i]) < 0) {
                printf("Error reading controller %d\n", player_count+1);
                break;
            }
        }


        if (!(conds[0].buttons & CONT_DPAD_DOWN))
            if (!(conds[0].buttons & CONT_B))
                vid_screen_shot( "/pc/home/jim/dc/dcscreenshot.ppm");

        /*** reboot condition: ABXYSTART ***/
        if (!(conds[0].buttons & CONT_START))
            if (!(conds[0].buttons & CONT_A))
                if (!(conds[0].buttons & CONT_B))
                    if (!(conds[0].buttons & CONT_X))
                        if (!(conds[0].buttons & CONT_Y))
                            break;


        if ( player_count >= 2 ) 
            if (!(conds[1].buttons & CONT_START))
                if (!(conds[1].buttons & CONT_A))
                    if (!(conds[1].buttons & CONT_B))
                        if (!(conds[1].buttons & CONT_X))
                            if (!(conds[1].buttons & CONT_Y))
                                break;


        /*
         *
         * the action of the frame 
         *
         *
         */
        /*** check player pauses and set runstate ***/
        running_state = RUNSTATE_PLAYING;
        for ( i=0; i<player_count; i++ ) {
            if ( players[i]->pausing == 1 ) {
                running_state = RUNSTATE_PAUSE;
            }
        }


        /*** consequences every frame ***/
        if ( running_state == RUNSTATE_PLAYING ) {
            /*** check controllers and take appropriate action ***/
            for ( i=0; i<player_count; i++ ) {
                process_input2( players[i], conds[i] );
                determine_fate( players[i] );

            }
            process_shots();
            move_asteroids();

#ifdef RANDOM_UFOS
            /* v0.0.6: random adding of enemies
             *
             *
             *
             */
            if ( ( rand() % MAX_RANDOM_PERCENTAGE ) < enemy_frequency ) {
                if ( enemy_count < MAX_ENEMIES ) {
                    enemies[ enemy_count ] = malloc( sizeof( enemy_t ) );
                    init_enemy( enemies[ enemy_count ] );
                    enemy_count++;
                }

            }
#endif

#ifdef RANDOM_POWERUPS
            /* v0.0.6: random adding of enemies
             *
             *
             *
             */
            if ( ( rand() % MAX_RANDOM_PERCENTAGE ) < powerup_frequency ) {
                if ( powerup_count < MAX_ENEMIES ) {
                    powerups[ powerup_count ] = malloc( sizeof( powerup_t ) );
                    init_powerup( powerups[ powerup_count ] );
                    powerup_count++;
                }

            }
#endif




        }
        /*** not playing ***/
        else {
            /*** check (both) controllers and take appropriate action ***/
            for ( i=0; i<player_count; i++ ) {
                process_prefs_input( players[i], conds[i] );
            }

        }






        /**** display code ***/
        glKosBeginFrame();
        


        glClearColor( 0.0, 0.0, 0.0, 0.0 );

 



        /* draw all active players' viewports/screens
         *
         *
         */
        for ( i=0; i<player_count; i++ ) {


            if ( player_count >= i+1 ) {
                float coeff = (float)(256 - players[i]->zoom)/256.0f;
                   
                glMatrixMode( GL_PROJECTION );
                glLoadIdentity();
                gluPerspective( (float)players[i]->fov_tight + 
                                coeff * (float)(players[i]->fov_wide - players[i]->fov_tight), 
                                (float)players[i]->screen->width / (float)players[i]->screen->height, 
                                0.01f, 1000.0f);
                glMatrixMode( GL_MODELVIEW );
                glViewport( players[i]->screen->x, players[i]->screen->y, 
                           players[i]->screen->width, players[i]->screen->height );
                glScissor( players[i]->screen->x, players[i]->screen->y, 
                           players[i]->screen->width, players[i]->screen->height );

                draw_scene( players[i] );



                /*** v0.0.6: draw player score ***/
                sprintf( pmes, "p%d: %d", i+1, players[i]->score );
                plprint( players[i]->screen->scoretext_x, players[i]->screen->scoretext_y, 
                         // 1.0f, 0.0f, 0.0f,
                         players[i]->color[0], 
                         players[i]->color[1], 
                         players[i]->color[2], 
                         pmes, 0);
                glKosPolyHdrDirty();

            }

        }




        /* dividing line 
         *
         *
         */
        glViewport( 0, 0, info_band->width, info_band->height );
        glScissor( info_band->x, info_band->y, info_band->width, info_band->height );
        /* player status HUD
         *
         *
         */
        sprintf( pmes, "level: %d  asteroids: %d  enemies: %d",
                 current_level,
                 asteroid_count,
                 enemy_count
            );
        plprint( info_band->messagetext_x, info_band->messagetext_y,
                 1.0f, 0.0f, 1.0f,
                 pmes, 0);








#ifdef FRAME_RATE_COUNTER
        /*** frame rate counter ***/
        if ( 1 ) {
            sprintf( pmes, "%.3f fps", frames_per_second );
            draw_string( 10.0, 10.0, 1.0,
                         1.0f, 1.0f, 1.0f, 1.0f,
                         pmes, 10 );	
        }
        frame_count++;
#endif







        /* win condition
         *
         *
         */
        if ( asteroid_count == 0 ) {
            int go_next = 0;
            running_state = RUNSTATE_DONE;

            sprintf( pmes, "level done - press A to advance" );
            draw_string( 100.0, 200.0, 1.0,
                         1.0f, 1.0f, 1.0f, 1.0f,
                         pmes, strlen(pmes) );	

            /*** somebody presses A ***/
            for ( i=0; i<player_count; i++ ) {
                if ( !(conds[i].buttons & CONT_A) ) { go_next = 1; }
            }
            if ( go_next >= 1 ) {
                current_level++;
                init_game_level( current_level );
            }

        }






        /* Finish the frame */
        glKosFinishFrame();


    } /*** end main loop ***/
  
#ifdef OGGVORBIS_BGM
    sndoggvorbis_stop();
    sndoggvorbis_shutdown();
    return 0;
#endif
}


